/*****************************************************************************/
/* Analyzer.h    Analyzer engine        (C)Copyright Gregory J. Shrago 1997  */
/*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#define TYPE_UNKNOWN    0      // Unknown type
#define TYPE_CONST      1      // Constant "10"
#define TYPE_VARIABLE   2      // Variable "x"\"X" only
#define TYPE_OPERATOR   3      // Operator "Left op. Right"
#define TYPE_FUNCTION   4      // Function "func (Left)"

#define OP_NONE         0      // None
#define OP_PLUS         1      // "+"
#define OP_MINES        2      // "-"
#define OP_MULTPLY      3      // "*"
#define OP_DIVIDE       4      // "/"
#define OP_POWER        5      // "^"


#define FUNC_NONE       0
#define FUNC_SIN        1
#define FUNC_COS        2
#define FUNC_TG         3
#define FUNC_ASIN       4
#define FUNC_ACOS       5
#define FUNC_ATG        6
#define FUNC_LN         7
#define FUNC_LG         8
#define FUNC_EXP        9
#define FUNC_ABS        10


#define CONST_PI  3.141592654
#define CONST_E   2.718281828
#define CONST_EPS 1e-10

/*****************************************************************************/
// Exception class
class XMathException
{
  public:
   XMathException(char* s, int T=0){strcpy(Name, s); Type=T;}

   char Name[64];
   int  Type;
};

/*****************************************************************************/
// XStringAnalyzer class
class XStringAnalyzer
{
  public:
    static int SIMPLIFY;
  
    XStringAnalyzer(char* s, int c=0);
    XStringAnalyzer(XStringAnalyzer* s);
    XStringAnalyzer(int T, XStringAnalyzer* L, XStringAnalyzer* R, double P);
    ~XStringAnalyzer();

    static char* SetupString(char* String); 
    static double Approximate(double v); 

    char* toString(char* s, int size);
    double Count(double x);

    XStringAnalyzer* Derivative();

    int isConst();
    int isEqual(XStringAnalyzer* s);
    double operator ()(double x){ return Count(x);}

  protected:
    XStringAnalyzer *Left;
    XStringAnalyzer *Right;

    int Type;
    double Param;

    XStringAnalyzer& DelParen(char* s);
    XStringAnalyzer& Analyze(char* s);
    XStringAnalyzer& Clone(XStringAnalyzer *s);

    XStringAnalyzer& SSimplify();
    XStringAnalyzer& SOrderOps();
    XStringAnalyzer& SDeOrderOps();
    XStringAnalyzer& SCountPeer();
    XStringAnalyzer& SCountConst();
    XStringAnalyzer** FindOperands(XStringAnalyzer** Buf, int Oper, int* size);
};
