/*****************************************************************************/
/* Solver.cpp  GUI for the analyzer.     (C)Copyright Gregory J. Shrago 1997 */
/*****************************************************************************/
#define INCL_PM
#define INCL_WIN
#define INCL_DOS
#define INCL_GPI
#define INCL_GPIBITMAPS
#define INCL_WINWORKPLACE
#include <os2.h>

#include "analyzer.h"
#include "resource.h"

typedef XStringAnalyzer* FUNCTION;
#define FFunc(F, x, Value) XStringAnalyzer::Approximate(F->Count(x)-Value);

/***********************************************************/
const char CopyRight[]="(C)Copyright Gregory J. Shrago. Russia 1997";
const char PrgmTitle[]= "Function Analyzer";

#define WM_UPDATE_STATUS (WM_USER+1)
#define ROOTS_DIFF 0.001
/***********************************************************/
struct POINTD {
  double x;
  double y;
};

struct SOLVESTRUCT
{
  FUNCTION F;
  double   Value;
  double   m, M;
  int      Count;
  double*  Res;
  FNTHREAD* SFunction;
  HWND     Progress;
};

HAB  hab;
HWND hwndMain;
PFNWP WOldProc;
BOOL MAKEENV = TRUE;
int  XAxis, YAxis,
     Width, Height;

double  XMIN = 0, XMAX = 0,
        YMIN = 0, YMAX = 0,
        Fp = 0;
int  FuncNum=0, FuncIndex=0;
XStringAnalyzer **Func = NULL;
XStringAnalyzer * Fx=NULL;

TID  TidStatus, TidSolve;


FNTHREAD StatusThread;
FNTHREAD SolveThread;
FNTHREAD IntegralThread;

MRESULT EXPENTRY ProdInfoDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
MRESULT EXPENTRY MainDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
MRESULT EXPENTRY GraphProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
MRESULT EXPENTRY SolveDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);

void MakeEnv(HWND hw, char* str,  int Refresh=0, int show=0);
void Plot(HWND hwnd, HPS ps, int index);

void SolveFunction(SOLVESTRUCT* SS, double m, double M, int Type=0);
BOOL AddSolution(SOLVESTRUCT* SS, double Val);

int Contain(POINTD*, int, int, int, int);
int Intersect(POINTD*, POINTD*, POINTD*, int, int, int, int);
/************************************************************/

char* FindFunction(char* String){
int i = strlen(String)-1;
  for(;i>-1;i--)
     if(String[i]=='=') break;
  for(i++;i<strlen(String);i++)
     if(String[i]!=' ') break;
return &String[i];
}

void main(VOID){
HMQ   hmq;
QMSG qmsg;
  hab = WinInitialize( 0 );
  hmq = WinCreateMsgQueue( hab, 0 );

  WinDlgBox( HWND_DESKTOP, HWND_DESKTOP, (PFNWP)ProdInfoDlgProc, NULL, ID_PRODINFO, NULL);
  hwndMain = WinLoadDlg(HWND_DESKTOP, HWND_DESKTOP, MainDlgProc, NULL, ID_MAINWINDOW, NULL);

  DosCreateThread(&TidStatus, StatusThread,0L,CREATE_READY,8192L);

  while( WinGetMsg( hab, &qmsg, (HWND)NULL, 0, 0 ))
    WinDispatchMsg( hab, &qmsg );

  DosKillThread(TidStatus);
  WinDestroyWindow( hwndMain );
  WinDestroyMsgQueue( hmq );
  WinTerminate( hab );
}

/******************************************************************/

void StatusThread(ULONG ulparm){
char Buf[128];
int l=strlen(CopyRight), i=0;
ULONG Cnt=0;
int Flag=0, Count=0;
 while(!(Count = 0)) {
   while(1) {
     sprintf(Buf, "%s", &CopyRight[i]);
     WinPostMsg(hwndMain, WM_UPDATE_STATUS, Buf, NULL);
     if(i++==0)DosSleep(6000);
     i=(i>l)?(0, Count++, DosSleep(6000)):i;
     if(Count == 5)break;
     DosSleep(50);
   }
   DosSleep(60000);
 }
}

/******************************************************************/

MRESULT EXPENTRY ProdInfoDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2){
   switch(msg){
     case WM_INITDLG:
          RECTL Desk, Dlg;
          WinQueryWindowRect(HWND_DESKTOP, &Desk);
          WinQueryWindowRect(hwnd, &Dlg);
          WinSetWindowPos(hwnd, HWND_TOP, (Desk.xRight-Desk.xLeft-Dlg.xRight+Dlg.xLeft)/2,
            (Desk.yTop-Desk.yBottom-Dlg.yTop+Dlg.yBottom)/2,0,0, SWP_MOVE);
          WinStartTimer(hab, hwnd, 1, 2000);
          break;
     case WM_TIMER:
          WinStopTimer(hab, hwnd, 1);
          WinDismissDlg(hwnd, TRUE);
          break;
     case WM_COMMAND:
          WinDismissDlg(hwnd, TRUE);
          break;
     default:
         return(WinDefDlgProc(hwnd, msg, mp1, mp2));
         break;
   }
return (MRESULT)0;
}

/******************************************************************/

MRESULT EXPENTRY MainDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2){
char Buf[1024], ResultStr[1024+50];
char b[256];
double Value;
int i;
SOLVESTRUCT* SS;
TID  TidSolve;
  switch(msg){
    case WM_INITDLG:
         RECTL Desk, Dlg;
         WinQueryWindowRect(HWND_DESKTOP, &Desk);
         WinQueryWindowRect(hwnd, &Dlg);
         if(!WinRestoreWindowPos("Analyzer", "Main", hwnd))
            WinSetWindowPos(hwnd, HWND_TOP, (Desk.xRight-Desk.xLeft-Dlg.xRight+Dlg.xLeft)/2,
              (Desk.yTop-Desk.yBottom-Dlg.yTop+Dlg.yBottom)/2,0,0, SWP_MOVE);

         WOldProc=WinSubclassWindow(WinWindowFromID(hwnd, ID_GRAPH), GraphProc);
         WinSendDlgItemMsg(hwnd, ID_RESULT,    EM_SETTEXTLIMIT, MPARAM(sizeof(ResultStr)), NULL);
         WinSendDlgItemMsg(hwnd, ID_PARAM,     EM_SETTEXTLIMIT, MPARAM(sizeof(Buf)), NULL);
         WinSendDlgItemMsg(hwnd, ID_FUNCCOMBO, EM_SETTEXTLIMIT, MPARAM(sizeof(Buf)), NULL);

         WinSendDlgItemMsg(hwnd, ID_SIMPLIFY, BM_SETCHECK, (MPARAM)XStringAnalyzer::SIMPLIFY, NULL);
         WinSetDlgItemText(hwnd, ID_XMIN, "-10");
         WinSetDlgItemText(hwnd, ID_XMAX, "10");
         WinSetDlgItemText(hwnd, ID_YMIN, "-10");
         WinSetDlgItemText(hwnd, ID_YMAX, "10");
         Fp = XMAX = XMIN = YMAX = YMIN = 0;
         MakeEnv(hwnd,"XxYyPF");
         break;
    case WM_UPDATE_STATUS:
         WinSetDlgItemText(hwnd, ID_STATUS, (char*)mp1);
         break;
    case WM_CONTROL:
         switch(SHORT1FROMMP(mp1)){
           case ID_FUNCCOMBO:
                if(SHORT2FROMMP(mp1)==CBN_LBSELECT)
                  MakeEnv(hwnd, "F");
                break;
           case ID_LIST:
                if(SHORT2FROMMP(mp1)==LN_SELECT)
                  MakeEnv(hwnd, "", 1);
                break;
           case ID_SIMPLIFY:
                if(SHORT2FROMMP(mp1)==BN_CLICKED)
                  XStringAnalyzer::SIMPLIFY=1-XStringAnalyzer::SIMPLIFY;
                break;
         }
         if(SHORT2FROMMP(mp1)==EN_CHANGE || SHORT2FROMMP(mp1)==CBN_EFCHANGE){
           MakeEnv(hwnd, "FPXxYy");
         }
         break;
    case WM_COMMAND:
         strcpy(ResultStr, "<Result> Can\'t perform the specified action");
         switch(SHORT1FROMMP(mp1)){
           case ID_COUNTAT:
//                MakeEnv(hwnd, "FP", 0, 1);
                if(Fx){
                  try {
                      Value = XStringAnalyzer::Approximate(Fx->Count(Fp));
                      sprintf(Buf, "F(");
                      sprintf(b, "%.6f", Fp);
                      strcat(Buf, XStringAnalyzer::SetupString(b));
                      strcat(Buf, ")= ");
                      sprintf(b, "%.6f", Value);
                      strcat(Buf, XStringAnalyzer::SetupString(b));
                  } catch(XMathException e){
                      sprintf(Buf, "Error %s", e.Name);
                  }
                  sprintf(ResultStr, "<CountAt> %s", Buf);
                }
                WinSetDlgItemText(hwnd, ID_RESULT, ResultStr);
                break;
           case ID_FUNC:
//                MakeEnv(hwnd, "F", 0, 1);
                if(Fx){
                  WinQueryDlgItemText(hwnd, ID_FUNCCOMBO, sizeof(Buf), Buf);
                  Func = (XStringAnalyzer**)realloc(Func, ++FuncNum*sizeof(void*));
                  Func[FuncNum-1] = Fx;
                  i=(int)WinSendDlgItemMsg(hwnd, ID_FUNCCOMBO, LM_QUERYITEMCOUNT, NULL, NULL);
                  sprintf(b, "F%d(x)", FuncIndex);
                  WinSendDlgItemMsg(hwnd, ID_LIST, LM_INSERTITEM, MPARAM(LIT_END), b);
                  sprintf(b,"F%d(x)= %s", FuncIndex++, FindFunction(Buf));
                  WinSendDlgItemMsg(hwnd, ID_FUNCCOMBO, LM_INSERTITEM, MPFROMLONG(i), b);
                  strcpy(Buf, "");
                  try {
                      Fx->toString(Buf, sizeof(Buf));
                  } catch (XMathException e) {}
                  sprintf(ResultStr, "<Add> F(x)= %s", Buf);
                  Fx = NULL;
                }
                WinSetDlgItemText(hwnd, ID_RESULT, ResultStr);
                break;
           case ID_PROCESS:
//                MakeEnv(hwnd, "F", 0, 1);
                if(Fx){
                  strcpy(Buf, "");
                  try {
                      Fx->toString(Buf, sizeof(Buf));
                  } catch (XMathException e) {}
                  sprintf(ResultStr, "<Process> F(x)= %s", Buf);
                }
                WinSetDlgItemText(hwnd, ID_RESULT, ResultStr);
                break;
           case ID_SOLVEAT:
//                MakeEnv(hwnd, "FPXx", 0, 1);
                if(Fx && (XMAX > XMIN)){
                  SS = new SOLVESTRUCT;
                  SS->F=Fx;
                  SS->M = XMAX;
                  SS->m = XMIN;
                  SS->Value= Fp;
                  SS->Count=0;
                  SS->Res = NULL;
                  SS->SFunction = SolveThread;
                  if(SS->F->isConst())
                  if(SS->F->Count(0)==SS->Value)
                    strcpy(Buf, "Number of roots is infinity");
                  else
                    strcpy(Buf, "Number of roots is zero");
                  else {
                    strcpy(Buf, "Can\'t find them (if they really exist)");
                    WinDlgBox( HWND_DESKTOP, hwnd, (PFNWP)SolveDlgProc, NULL, ID_PROGRESSDLG, (PVOID)SS);
                    if(SS->Count){
                      sprintf(Buf, "%d root(s): ", SS->Count);
                      for(i=0; i<SS->Count; i++){
                         sprintf(b, " x%d=%.6f", i, XStringAnalyzer::Approximate(SS->Res[i]));
                         strcat(Buf, XStringAnalyzer::SetupString(b));
                      }
                      free(SS->Res);
                    }
                  }
                  delete SS;
                  sprintf(ResultStr, "<SolveAt> %s", Buf);
                }
                WinSetDlgItemText(hwnd, ID_RESULT, ResultStr);
                break;
           case ID_DERIVATIVE:
//                MakeEnv(hwnd, "F", 0, 1);
                if(Fx){
                  WinQueryDlgItemText(hwnd, ID_FUNCCOMBO, sizeof(Buf), Buf);
                  try {
                      XStringAnalyzer* Temp = Fx->Derivative();
                      sprintf(Buf,"F'(x)= ");
                      try {
                          Temp->toString(&Buf[strlen(Buf)], sizeof(Buf)-strlen(Buf));
                      } catch (XMathException e) {}
                      sprintf(ResultStr, "<Derivative> %s", Buf);
                      delete Temp;
                  } catch (XMathException e){
                      sprintf(Buf, "Derivative cannot be found. %s", e.Name);
                  }
                }
                WinSetDlgItemText(hwnd, ID_RESULT, ResultStr);
                break;
           case ID_INTEGRAL:
//                MakeEnv(hwnd, "FPXx", 0, 1);
                if(Fx && (XMAX > XMIN)){
                  SS = new SOLVESTRUCT;
                  SS->F=Fx;
                  SS->M = XMAX;
                  SS->m = XMIN;
                  SS->Value= Fp;
                  SS->Count=0;
                  SS->Res = NULL;
                  SS->SFunction = IntegralThread;
                  strcpy(Buf, "Integral cannot be found");
                  WinDlgBox( HWND_DESKTOP, hwnd, (PFNWP)SolveDlgProc, NULL, ID_PROGRESSDLG, (PVOID)SS);
                  if(SS->Count){
                    sprintf(Buf, "Int(");
                    sprintf(b, "%.6f", SS->m);
                    strcat(Buf, XStringAnalyzer::SetupString(b));
                    strcat(Buf, ", ");
                    sprintf(b, "%.6f", SS->M);
                    strcat(Buf, XStringAnalyzer::SetupString(b));
                    strcat(Buf, ")= ");
                    sprintf(b, "%.6f", XStringAnalyzer::Approximate(SS->Res[0]));
                    strcat(Buf, XStringAnalyzer::SetupString(b));
                    free(SS->Res);
                    delete SS;
                  }
                  sprintf(ResultStr, "<Integral> %s", Buf);
                }
                WinSetDlgItemText(hwnd, ID_RESULT, ResultStr);
                break;
           case ID_REMOVE:
                if(WinMessageBox(HWND_DESKTOP, hwnd, "Remove all selected functions?",
                                 PrgmTitle, 20, MB_YESNO | MB_QUERY | MB_DEFBUTTON2
                                 | MB_APPLMODAL | MB_MOVEABLE)==MBID_YES){
                  int N=LIT_FIRST;
                  while((N=(int)WinSendDlgItemMsg(hwnd, ID_LIST, LM_QUERYSELECTION, MPFROMLONG(N), NULL))!=LIT_NONE){
                    for(i=0;i<FuncNum;i++)
                       if(i==N){
                         delete Func[i];
                         WinSendDlgItemMsg(hwnd, ID_LIST, LM_DELETEITEM, MPFROMLONG(N), NULL);
                         for(int j=N; j<FuncNum-1; j++)
                            Func[j]=Func[j+1];
                         FuncNum--;  
                       }
                  }
                  Func=(XStringAnalyzer**)realloc(Func, FuncNum*sizeof(XStringAnalyzer*));
                  MakeEnv(hwnd, "", 1);
                }
                break;
           case ID_REFRESH:
                MakeEnv(hwnd, "XxYy", 1);
                break;
         }
         break;
    case WM_CLOSE:
         for(i=0; i< FuncNum; i++) delete Func[i];
         if(FuncNum) free(Func);
         if(Fx) delete Fx;
         WinStoreWindowPos("Analyzer", "Main", hwnd);
         WinPostMsg(hwnd, WM_QUIT, NULL, NULL);
         break;
    default:
         return(WinDefDlgProc(hwnd, msg, mp1, mp2));
         break;
   }
return (MRESULT)0;
}

void MakeEnv(HWND hwnd, char* str, int Refresh, int show)
{
  int i, ID=0, P=1, X=1, Y=1, D=1;
  double *Addr;
  char Buf[1024];
//  Fp = XMAX = XMIN = YMAX = YMIN = 0;
  if(!MAKEENV) return;
  for(i=0;i<strlen(str);i++){
     switch(str[i]){
       case 'F': ID=0;
                 if(Fx){
                   delete Fx;
                   Fx=NULL;
                 }
                 WinQueryDlgItemText(hwnd, ID_FUNCCOMBO, sizeof(Buf), Buf);
                 if(!strlen(Buf)) break;
                 try {
                     Fx = new XStringAnalyzer(FindFunction(Buf));
                     if(strstr(Buf, "abs"))D=0;
                 } catch (XMathException e) {
                     Fx=NULL;
                     if(show){
                       Box(PrgmTitle, e.Name);
                       break; 
                     }
                 }
                 break;
       case 'P': ID=ID_PARAM; Addr=&Fp; break;
       case 'X': ID=ID_XMAX;  Addr=&XMAX;  break;
       case 'x': ID=ID_XMIN;  Addr=&XMIN;  break;
       case 'Y': ID=ID_YMAX;  Addr=&YMAX;  break;
       case 'y': ID=ID_YMIN;  Addr=&YMIN;  break;
     }
     if(ID){
       *Addr=0;
       try {
         WinQueryDlgItemText(hwnd, ID, sizeof(Buf), Buf);
        *Addr = XStringAnalyzer::Approximate(XStringAnalyzer(Buf, 1)(0));
       } catch (XMathException e) {
         if(ID==ID_PARAM) P=0;
         if(ID==ID_XMIN || ID==ID_XMAX) X=0;
         if(ID==ID_XMIN || ID==ID_XMAX) Y=0;
         if(show) {
           Box(PrgmTitle, e.Name);
           break;
         }
       }
     }
  }
  if(!X) XMIN=XMAX;
  if(!Y) YMIN=YMAX;
  WinEnableWindow(WinWindowFromID(hwnd, ID_FUNC),       (Fx)?1:0);
  WinEnableWindow(WinWindowFromID(hwnd, ID_PARAM),      (Fx)?1:0);
  WinEnableWindow(WinWindowFromID(hwnd, ID_COUNTAT),    (Fx && P)?1:0);
  WinEnableWindow(WinWindowFromID(hwnd, ID_PROCESS),    (Fx)?1:0);
  WinEnableWindow(WinWindowFromID(hwnd, ID_SOLVEAT),    ((XMAX > XMIN) && Fx && P)?1:0);
  WinEnableWindow(WinWindowFromID(hwnd, ID_INTEGRAL),   ((XMAX > XMIN) && (Fp>0) && Fx)?1:0);
  WinEnableWindow(WinWindowFromID(hwnd, ID_DERIVATIVE), (Fx && D)?1:0);
  WinEnableWindow(WinWindowFromID(hwnd, ID_LIST),       ((XMAX > XMIN) && (YMAX > YMIN) && FuncNum)?1:0);
  WinEnableWindow(WinWindowFromID(hwnd, ID_XMAX),       (Fx || FuncNum)?1:0);
  WinEnableWindow(WinWindowFromID(hwnd, ID_XMIN),       (Fx || FuncNum)?1:0);
  WinEnableWindow(WinWindowFromID(hwnd, ID_YMAX),       (FuncNum)?1:0);
  WinEnableWindow(WinWindowFromID(hwnd, ID_YMIN),       (FuncNum)?1:0);
  WinEnableWindow(WinWindowFromID(hwnd, ID_REFRESH),    (FuncNum)?1:0);

  if(!((XMAX > XMIN) && (YMAX > YMIN) && FuncNum) && Refresh) {
    WinSendDlgItemMsg(hwnd, ID_LIST, LM_SELECTITEM, MPARAM(LIT_NONE), FALSE);
//    WinInvalidateRect(WinWindowFromID(hwnd, ID_GRAPH), 0, 0);
  } else if(Refresh)
;//    WinInvalidateRect(WinWindowFromID(hwnd, ID_GRAPH), 0, 0);
  WinInvalidateRect(WinWindowFromID(hwnd, ID_GRAPH), 0, 0);
  WinEnableWindow(WinWindowFromID(hwnd, ID_REMOVE),    ((int)WinSendDlgItemMsg(hwnd, ID_LIST, LM_QUERYSELECTION, MPFROMLONG(LIT_FIRST), NULL)!=LIT_NONE)?1:0);
return;
}


MRESULT EXPENTRY SolveDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
char Buf[64];
HWND hOwner;
SOLVESTRUCT* SS;
int Progress;
   switch(msg){
     case WM_INITDLG:
          RECTL Desk, Dlg, Owner;
          SS = (SOLVESTRUCT*)mp2;
          hOwner = WinQueryWindow(hwnd, QW_OWNER);
          WinQueryWindowRect(HWND_DESKTOP, &Desk);
          WinQueryWindowRect(hOwner, &Owner);
          WinQueryWindowRect(hwnd, &Dlg);
          WinSetWindowPos(hwnd, HWND_TOP, Owner.xLeft+(Desk.xRight-Desk.xLeft-Dlg.xRight+Dlg.xLeft)/2,
            Owner.yBottom+(Desk.yTop-Desk.yBottom-Dlg.yTop+Dlg.yBottom)/2,0,0, SWP_MOVE);
          SS->Progress=hwnd;
          DosCreateThread(&TidSolve, SS->SFunction,(ULONG)SS,CREATE_READY,32768L);
          break;
     case WM_UPDATE_STATUS:
          Progress = (int)mp1;
          sprintf(Buf, "Counting: %d %% completed", Progress);
          if(Progress > 100) strcat(Buf, " :)");
          WinSetDlgItemText(hwnd, ID_STATUS, Buf);
          WinSendDlgItemMsg(hwnd,ID_PROGRESS, SLM_SETSLIDERINFO, MPFROM2SHORT
             (SMA_SLIDERARMPOSITION,SMA_INCREMENTVALUE),MPFROMSHORT(Progress));

          if(Progress >= 100)
            WinSetDlgItemText(hwnd, ID_SPBUTTON, "~Ok");
          break;
     case WM_COMMAND:
          DosKillThread(TidSolve);
          WinDismissDlg(hwnd, TRUE);
          break;
     default:
          return(WinDefDlgProc(hwnd, msg, mp1, mp2));
          break;
   }
return (MRESULT)0;
}


MRESULT EXPENTRY GraphProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
static int    Drag=0;
static POINTL Begin, End;
double _XMAX, _YMAX, _XMIN, _YMIN;

HPS hps;
RECTL R;
POINTL P;
int i=LIT_FIRST;

 HPS hpsM;
 HDC hdcM;
 BITMAPINFOHEADER2 BmpHdr;
 HBITMAP hBmp;
 DEVOPENSTRUC dop ={NULL, "DISPLAY", NULL, NULL, NULL, NULL, NULL, NULL, NULL};
 SIZEL szl={0,0};
 POINTL Points[3]={0,0,0,0,0,0};

 switch(msg){
    case WM_PAINT:
         WinQueryWindowRect(hwnd, &R);
         Width = R.xRight;
         Height = R.yTop;

         szl.cx = Width; szl.cy = Height;
         hdcM=DevOpenDC(hab,OD_MEMORY,"*",3L,(PDEVOPENDATA)&dop, NULLHANDLE);
         hpsM=GpiCreatePS(hab,hdcM,&szl, GPIA_ASSOC|PU_PELS);
         BmpHdr.cbFix=sizeof(BITMAPINFOHEADER2);
         BmpHdr.cx=Width;
         BmpHdr.cy=Height;
         BmpHdr.cPlanes=1;
         BmpHdr.cBitCount=16;
         hBmp=GpiCreateBitmap(hpsM,&BmpHdr,0L,NULL,NULL);
         GpiSetBitmap(hpsM,hBmp);

         hps = WinBeginPaint(hwnd, NULLHANDLE, NULL);
         GpiSetColor(hpsM, CLR_BLACK);
         GpiSetBackColor(hpsM, CLR_DARKGRAY);
         WinFillRect(hpsM, &R, CLR_DARKGRAY);

         XAxis = Height/2;
         YAxis = Width/2;

         GpiSetLineWidth(hpsM, LINEWIDTH_NORMAL);
         if((XMAX > XMIN) && (YMAX > YMIN) && FuncNum){
             XAxis = -YMIN*Height/(YMAX-YMIN);
             YAxis = -XMIN*Width/(XMAX-XMIN);
             while((i=(int)WinSendDlgItemMsg(hwndMain, ID_LIST, LM_QUERYSELECTION, MPFROMLONG(i), NULL))!=LIT_NONE)
                   Plot(hwnd, hpsM, i);
         }

         GpiSetLineWidth(hpsM, LINEWIDTH_THICK);

         P.x = YAxis; P.y = 0;      GpiMove(hpsM, &P);
         P.x = YAxis; P.y = Height; GpiLine(hpsM, &P);
         P.x = 0; P.y = XAxis;      GpiMove(hpsM, &P);
         P.x = Width; P.y = XAxis;  GpiLine(hpsM, &P);

         Points[1].x = Width; Points[1].y = Height;
         GpiBitBlt(hps, hpsM, 3, Points, ROP_SRCCOPY, BBO_IGNORE);
         
         GpiSetBitmap(hpsM, NULLHANDLE);
         GpiDeleteBitmap(hBmp);
         GpiDestroyPS(hpsM);
         DevCloseDC(hdcM);

         WinEndPaint(hps);
         break;
    case WM_BUTTON1DOWN:
    case WM_BUTTON2DOWN:
         if((XMAX > XMIN) && (YMAX > YMIN) && FuncNum){
           Drag=1;
           Begin.x = End.x = SHORT1FROMMP(mp1);
           Begin.y = End.y = SHORT2FROMMP(mp1);
           WinSetCapture(HWND_DESKTOP, hwnd);
         }
         break;
    case WM_BUTTON1UP:
    case WM_BUTTON2UP:
         if(Drag){
           WinSetCapture(HWND_DESKTOP, NULLHANDLE);
           if(Drag==1){
             Drag=0;  
             break;
           }
           Drag=0;

           WinQueryWindowRect(hwnd, &R);
           Width = R.xRight;
           Height = R.yTop;

           if(Begin.x>End.x){
             i = Begin.x; Begin.x = End.x; End.x = i; }
           if(Begin.y>End.y){
             i = Begin.y; Begin.y = End.y; End.y = i; }

           if(End.x - Begin.x < 5 || End.y - Begin.y<5) break;

           if(msg==WM_BUTTON1UP){
             _XMIN = (Begin.x-YAxis)*(XMAX-XMIN)/Width;
             _YMIN = (Begin.y-XAxis)*(YMAX-YMIN)/Height;
             _XMAX = (End.x-YAxis)*(XMAX-XMIN)/Width;
             _YMAX = (End.y-XAxis)*(YMAX-YMIN)/Height;
           } else {
             _XMIN = (XMIN*End.x-XMAX*Begin.x)/(End.x-Begin.x);
             _YMIN = (YMIN*End.y-YMAX*Begin.y)/(End.y-Begin.y);
             _XMAX = (XMIN*End.x-XMAX*Begin.x+Width*(XMAX-XMIN))/(End.x-Begin.x);
             _YMAX = (YMIN*End.y-YMAX*Begin.y+Height*(YMAX-YMIN))/(End.y-Begin.y);
           }
           char Buf[32];
           MAKEENV = FALSE;
           sprintf(Buf, "%f", _XMAX);
           WinSetDlgItemText(hwndMain, ID_XMAX, Buf);
           sprintf(Buf, "%f", _XMIN);
           WinSetDlgItemText(hwndMain, ID_XMIN, Buf);
           sprintf(Buf, "%f", _YMAX);
           WinSetDlgItemText(hwndMain, ID_YMAX, Buf);
           sprintf(Buf, "%f", _YMIN);
           MAKEENV=TRUE;
           WinSetDlgItemText(hwndMain, ID_YMIN, Buf);
         }
         break;
    case WM_MOUSEMOVE:
         if(!Drag) break;

         WinQueryWindowRect(hwnd, &R);
         Width = R.xRight;
         Height = R.yTop;
         hps = WinGetPS(hwnd);
         GpiSetColor(hps, CLR_DARKGRAY);
         GpiSetMix(hps, FM_XOR);
         GpiMove(hps, &Begin);
         if(abs(End.x-Begin.x)>5 && abs(End.y-Begin.y)>5){
           GpiBox(hps, DRO_OUTLINE, &End, 0, 0);
         }
         
         End.x = (SHORT)SHORT1FROMMP(mp1);
         End.y = (SHORT)SHORT2FROMMP(mp1);

         if(End.x<0) End.x=0;
           else if(End.x>Width)  End.x=Width;
         if(End.y<0) End.y=0;
           else if(End.y>Height) End.y=Height;

         if(abs(End.x-Begin.x)>5 && abs(End.y-Begin.y)>5){
           GpiBox(hps, DRO_OUTLINE, &End, 0, 0);
           Drag=2;
         } else Drag = 1;
         GpiDestroyPS(hps);
         break;
 }
 return WOldProc(hwnd, msg, mp1, mp2);
}

int Contain(POINTD* p, int a, int b, int c, int d) {
  if(p->x<a || p->y<b || p->x>c || p->y>d) return -1;
  if(p->x>a && p->y>b && p->x<c && p->y<d) return  1;
return 0;
}

int Intersect(POINTD* p1, POINTD* p2, POINTD* p3, int a, int b, int c, int d) {
double t=.5;
int f=0;
  if(fabs(p1->x-p2->x)<1) f+=1;
  if(fabs(p1->y-p2->y)<1) f+=2;
  if(f==0 || f==3) {
    t=.5;
  } else if(f==2) {
    t=(a-p2->x) / (p1->x-p2->x);
    if(t<0 || t>1) t=(c-p2->x)/(p1->x-p2->x);
    if(t<0 || t>1) return 0;
  } else if(f==1) {
    t=(b-p2->y) / (p1->y-p2->y);
    if(t<0 || t>1) t=(d-p2->y)/(p1->y-p2->y);
    if(t<0 || t>1) return 0;
  }
  p3->x = (p1->x-p2->x)*t+p2->x;
  p3->y = (p1->y-p2->y)*t+p2->y;
return 1;
}

void Plot(HWND hwnd, HPS ps, int index)
{
double x, y, Delta, Yk, Xk, t;
POINTD Point, Prev, Tmp;
POINTL P;
int Flag=0, Vis;

  Yk = Height/(YMAX-YMIN);
  Xk = Width/(XMAX-XMIN);
  Delta=1/Xk/10;

  GpiSetColor(ps, CLR_BLACK);
  for(x=XMIN, Vis=0; x <= XMAX; x+=Delta, Vis=0){
      try {
          y = Func[index]->Count(x);
      } catch(XMathException e) {
          Flag=0;
          continue;
      }
      Point.x = Xk*x + YAxis; Point.y = Yk*y + XAxis;
      if(Flag) {
        if(Contain(&Point, 0, 0, Width, Height)<0) Vis+=1;
        if(Contain(&Prev, 0, 0, Width, Height)<0) Vis+=2;
        if(Vis && Vis<3) {
          Intersect(&Point, &Prev, &Tmp, 0, 0, Width, Height);
          if(Vis==1) { P.x = Prev.x;  P.y=Prev.y; }
          if(Vis==2) { P.x = Point.x; P.y=Point.y; }
          GpiMove(ps, &P);
          P.x = Tmp.x;  P.y = Tmp.y;
          GpiLine(ps, &P);
        } else if(!Vis) {
          P.x = Prev.x;  P.y=Prev.y;
          GpiMove(ps, &P);
          P.x = Point.x; P.y=Point.y;
          GpiLine(ps, &P);
        }
      } else {
        Flag=1;
      }
      Prev=Point;
  }
}

/***************************************************/
void IntegralThread(ULONG parm)
{
int i;
SOLVESTRUCT* SS;
double Result=0;
double x, y, y0, Delta;
double Iter, Pr=0, Last=-1;
  SS = (SOLVESTRUCT*)parm;
  Delta = SS->Value;
  Iter = Delta/(SS->M - SS->m)*100;

  x=SS->m;
  try { 
      y0 = (*SS->F)(x); 
  } catch(XMathException e) {
      WinPostMsg(SS->Progress, WM_UPDATE_STATUS, MPFROMLONG(100), NULL);
      return;
  }
  for(x+=Delta; x<=SS->M; x+=Delta, Pr+=Iter) {
     if(fabs(Pr-Last)>1){
           WinPostMsg(SS->Progress, WM_UPDATE_STATUS, MPFROMLONG(Pr), NULL);
           Last = (int)Pr;
     }
     try {
         y = (*SS->F)(x); 
     } catch(XMathException e) {
         WinPostMsg(SS->Progress, WM_UPDATE_STATUS, MPFROMLONG(100), NULL);
         return; 
     }

     Result+=Delta/2*(y+y0);
     y0=y;
  }
  SS->Res = (double*)malloc(sizeof(double));
  SS->Res[SS->Count++]=Result;
  WinPostMsg(SS->Progress, WM_UPDATE_STATUS, MPFROMLONG((Pr<100)?100:Pr), NULL);
return;
}
/***************************************************/
void SolveThread(ULONG parm)
{
int i;
SOLVESTRUCT* SS;
double x, y, Delta;

int Last=-2;
double Iter, Pr=0;
  SS = (SOLVESTRUCT*)parm;

  Delta = (SS->M - SS->m)/100;
  Iter  = 1; /*Delta/(SS->M - SS->m)*100;*/

  for(x=SS->m; x<SS->M; x+=Delta, Pr+=Iter){
     if(fabs(Pr-Last)>1){
           WinPostMsg(SS->Progress, WM_UPDATE_STATUS, MPFROMLONG(Pr), NULL);
           Last = (int)Pr;
     }
     SolveFunction(SS, x, x+Delta);
  }
  WinPostMsg(SS->Progress, WM_UPDATE_STATUS, MPFROMLONG(Pr-Iter/*(Pr<100)?100:Pr*/), NULL);
return;
}


BOOL AddSolution(SOLVESTRUCT* SS, double Val)
{
int i=0;
  if(SS->Count<30){
    for(i=0; i<SS->Count; i++)
       if(fabs(Val-SS->Res[i])<ROOTS_DIFF){
         Val = XStringAnalyzer::Approximate((SS->Res[i]+Val)/2);
         SS->Res[i]=Val;
         return FALSE;
       }
    SS->Res = (double*)realloc(SS->Res, ++SS->Count*sizeof(double));
    SS->Res[i]=Val;
    return TRUE;
  }
return FALSE;
}

void SolveFunction(SOLVESTRUCT* SS, double m, double M, int Type)
{
double x, y, Delta;
double y0=0, dy=0;
int Flag=1, Accel=0;
/******************************************************/
  if(XStringAnalyzer::Approximate(M-m)==0){
    for(x=m, Delta=(M-m)/100; x<M;x+=Delta){
       try {
           y = FFunc(SS->F, x, SS->Value);
         } catch(XMathException e) { y=10; }
         if(fabs(y)<ROOTS_DIFF)
           AddSolution(SS, x);
         return;
      }
  }
/******************************************************/
  for(x=m, Delta=(M-m)/1000; x<=M; x+=Delta){
     try { 
         y = (*SS->F)(x); 
     } catch(XMathException e){
         if(!Flag)
           if(x+2*Delta<=M){ Delta*=2; Accel++;}
           else { Delta/=pow(2, Accel);  Accel = 0; }
            if(Flag)
              Flag=0;
            continue;
     }
        break;
  }
/******************************************************/
  for(; x<=M; x+=Delta){
     try { 
         y = FFunc(SS->F, x, SS->Value);
     } catch(XMathException e) {
         if(Flag) {
           Flag=0;
           SolveFunction(SS, x-Delta, x);
         } else {
           if(x+2*Delta<=M){ Delta*=2; Accel++;}
           else { Delta/=pow(2, Accel);  Accel = 0; }
         }
         continue;
     }
     if(!Flag){
       Flag=1;
       y0=y; dy=0;
       SolveFunction(SS, x-Delta, x);
       Delta/=pow(2, Accel);
       Accel = 0;
       continue;
     }
     if(y==0){
       AddSolution(SS, x);
       continue;
     }
     if(fabs(y+y0)<fabs(y)+fabs(y0)){
       SolveFunction(SS, x-Delta, x);
//            if(Type>2) return;
     }
     if(fabs(dy+(y-y0))<fabs(dy)+fabs(y-y0)){
       SolveFunction(SS, x-2*Delta, x);
//            if(Type>2) return;
     }
     dy=y-y0; y0=y;
  }
}
